## Introduction
This document serves to understand how to set up edgemark cluster given
that a base cluster (to run hollow-edge-node pods) and
separate master (to act as master for the hollow edge nodes) are already present.

## Precondition
You need edgemark master and external cluster to set up a edgemark cluster.

The functions are as follows:

- edgemark master: can be StandAlone or HA, used to be the edgemark cluster's master
- external cluster: used to create hollow edge nodes for the edgemark cluster

## Steps:
1. Deploy CloudCore in edgemark cluster, refer to [CloudCore deploy](https://kubeedge.io/en/docs/setup/ha/) for more details.

2. Build edgemark image

If you want to build/use your own edgemark image, do as follows.

- i. pull kubeedge code

```
cd $GOPATH/src/github.com/kubeedge
git clone git@github.com:kubeedge/kubeedge.git
```

- ii. build edgemark image

```
cd $GOPATH/src/github.com/kubeedge/kubeedge
make image WHAT=edgemark
```

Then you can get the image named `kubeedge/edgemark:{tag}` locally.

3. Create hollow nodes in external cluster

- i. create namespace and secret

Copy edgemark master's `tokensecret` which is generated by CloudCore which is used to access CloudCore for edge node,
and create it in namespace that will deploy hollow edge node in external cluster.

first, save `tokensecret` in file from edgemark cluster,

```
kubectl get secret -nkubeedge tokensecret -oyaml > tokensecret.yaml
```

modify namespace in `tokensecret.yaml`
```
sed -i "s|namespace: .*|namespace: {ns}|g" tokensecret.yaml
```

create ns and tokensecret in external cluster

```
kubectl create ns edgemark

kubectl create -f tokensecret.yaml
```

- ii. apply yaml to create hollow nodes

You can use `hollow-edge-node_template.yaml` in the current directory.

Note:

- the parameters `{{numreplicas}}` means the number of hollow nodes in the edgemark cluster
- the parameters `{{server}}` means the server address exposed for hollow nodes join in
- the parameters `{{server}}`, `{{numreplicas}}`, `{{edgemark_image_registry}}` and `{{edgemark_image_tag}}` need to be filled in the template
- your external cluster should have enough resources to be able to run `{{numreplicas}}` no. of hollow-node pods

```
kubectl create -f hollow-edge-node_template.yaml
```

Waiting for these hollow-node pods to be running. Then you can see these pods register as edgemark master's nodes.

Finally, edgemark master and external cluster set up the edgemark cluster.


4. Run performance testing with ClusterLoader2

After set up the edgemark cluster, we can do our performance testing with ClusterLoader2.
[ClusterLoader2](https://github.com/kubernetes/perf-tests/tree/master/clusterloader2) is
an official K8s scalability and performance testing framework.
refer to [ClusterLoader2 Getting started](https://github.com/kubernetes/perf-tests/blob/master/clusterloader2/docs/GETTING_STARTED.md) for more details.
we just need config `--provider=kubemark` when we run performance test, for example:

```
./clusterloader  --testconfig=config.yaml --provider=kubemark --kubeconfig=${HOME}/.kube/config --v=2
```




